/******************************************************************************
 * File: AdjacentRange.h
 * Date: 13.3.2018
 *****************************************************************************/

#ifndef ADJACENTRANGE_H
#define ADJACENTRANGE_H

#include <iterator>
#include <utility>

namespace grndr
{
	template <typename FwdIt>
	class AdjacentIterator
	{
	public:
		AdjacentIterator(FwdIt first, FwdIt last) : _first(first), _next(first == last ? first : std::next(first)) { }

		bool operator !=(const AdjacentIterator& other) const
		{
			return _next != other._next;
		}

		AdjacentIterator& operator++()
		{
			++_first;
			++_next;
			return *this;
		}

		using Ref = typename std::iterator_traits<FwdIt>::reference;
		using Pair = std::pair<Ref, Ref>;

		Pair operator*() const
		{
			return Pair(*_first, *_next);
		}

	private:
		FwdIt _first;
		FwdIt _next;
	};

	template <typename FwdIt>
	class AdjacentRange
	{
	public:
		AdjacentRange(FwdIt first, FwdIt last) : _first(first), _last(last) { }

		AdjacentIterator<FwdIt> begin() const
		{
			return AdjacentIterator<FwdIt>(_first, _last);
		}

		AdjacentIterator<FwdIt> end() const
		{
			return AdjacentIterator<FwdIt>(_last, _last);
		}

	private:
		FwdIt _first;
		FwdIt _last;
	};

	template <typename C>
	auto MakeAdjacentRange(C& c)
	{
		return AdjacentRange<decltype(c.begin())>(c.begin(), c.end());
	}
}

#endif
