/******************************************************************************
 * File: MRUStringList.h
 * Date: 01.3.2018
 *****************************************************************************/

#ifndef MRUSTRINGLIST_H
#define MRUSTRINGLIST_H

#include <QStringList>

namespace grndr
{
	class MRUStringList
	{
	public:
		MRUStringList(bool caseSensitive = true, bool pathList = false, unsigned int entryLimit = 0);
		MRUStringList(const MRUStringList& otherList) = default;
		MRUStringList(MRUStringList&& otherList) = default;
		MRUStringList(const QStringList& otherList) { _entries = otherList; purgeList(); }
		MRUStringList(QStringList&& otherList) { _entries = std::move(otherList); purgeList(); }

		MRUStringList& operator =(const MRUStringList& otherList) = default;
		MRUStringList& operator =(MRUStringList&& otherList) = default;
		MRUStringList& operator =(const QStringList& otherList) { _entries = otherList; purgeList(); return *this; }
		MRUStringList& operator =(QStringList&& otherList) { _entries = std::move(otherList); purgeList(); return *this; }

		operator QStringList() const { return _entries; }

	public:
		void add(QString entry);
		void remove(QString entry);		

		void operator +=(QString entry) { add(entry); }
		void operator -=(QString entry) { remove(entry); }

	public:
		auto at(int index) { return _entries.at(index); }
		bool contains(QString entry) const { return find(entry) != _entries.cend(); }

		auto size() { return _entries.size(); }
		auto empty() { return _entries.isEmpty(); }

		auto begin() { return _entries.begin(); }
		auto cbegin() const { return _entries.cbegin(); }
		auto end() { return _entries.end(); }
		auto cend() { return _entries.cend(); }

	public:
		void setCaseSensitive(bool caseSensitive = true) { _isCaseSensitive = caseSensitive; }
		void setPathList(bool pathList = true) { _isPathList = pathList; }
		void setEntryLimit(unsigned int limit) { _entryLimit = limit; purgeList(); }

	private:
		QStringList::iterator find(QString entry) { return find(entry, _entries.begin(), _entries.end()); }
		QStringList::const_iterator find(QString entry) const;
		template<typename ContainerType>
		ContainerType find(QString entry, ContainerType begin, ContainerType end);

		void purgeList();

	private:
		QStringList _entries;

		bool _isCaseSensitive{true};
		bool _isPathList{false};

		unsigned int _entryLimit{0};
	};
}

#include "MRUStringList.impl.h"

#endif
